//--------------------------------------------------------------
// File     : stm32_ub_sram.c
// Datum    : 16.07.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.0
// Module   : GPIO, FSMC
// Funktion : Externes SRAM [512 kByte]
//            Typ = AS7C34098A [4Mbit, 256k x 16bit, 10ns]
//            Der Zugriff erfolgt ueber den FSMC-Controller
//              Weil A0-A15 im 100Pin-Package nicht verfgbar ist,
//              wird im Adress-Multiplex-Mode gearbeitet (Mode-2)
//              und externe Adress-Latches sind notwendig
//
// Hinweis  : Das SRAM benutzt die CPU-Pins :
//             PD0 =RAM_AD2        PB7 =Latch_Enable (Lo-Aktiv)
//             PD1 =RAM_AD3        PE7 =RAM_AD4
//             PD4 =RAM_OE         PE8 =RAM_AD5
//             PD5 =RAM_WE         PE9 =RAM_AD6
//             PD7 =RAM_CE         PE10=RAM_AD7
//             PD8 =RAM_AD13       PE11=RAM_AD8
//             PD9 =RAM_AD14       PE12=RAM_AD9
//             PD10=RAM_AD15       PE13=RAM_AD10
//             PD11=RAM_A16        PE14=RAM_AD11
//             PD12=RAM_A17        PE15=RAM_AD12
//             PD14=RAM_AD0
//             PD15=RAM_AD1
//
// Speed: Read  = ca. 5,8 MByte/sec
//        Write = ca. 6,8 MByte/sec
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_sram.h"



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_SRAM_InitIO(void);
void P_SRAM_InitFSMC(void);





//--------------------------------------------------------------
// Init vom externen SRAM
// Return_wert :
//  -> ERROR   , wenn SRAM nicht gefunden wurde
//  -> SUCCESS , wenn SRAM gefunden wurde
//--------------------------------------------------------------
ErrorStatus UB_SRAM_Init(void)
{
  ErrorStatus ret_wert=ERROR;
  uint16_t oldwert,istwert;

  // IO-Lines initialisieren
  P_SRAM_InitIO();
  // FSMC initialisieren
  P_SRAM_InitFSMC();

  //-----------------------------------------
  // check ob SRAM vorhanden ist
  // schreib-/lese-Test auf Adr 0x00
  //-----------------------------------------
  oldwert=UB_SRAM_Read(0x00);
  UB_SRAM_Write(0x00,0x5A3C);
  istwert=UB_SRAM_Read(0x00);
  UB_SRAM_Write(0x00,oldwert);  
  if(istwert==0x5A3C) ret_wert=SUCCESS; // RAM vorhanden

  return(ret_wert);
}


//--------------------------------------------------------------
// schreibt einen 16bit Wert ins externe SRAM
// in eine Adresse
// wert : [0 bis 65535]
// adr  : [0 bis SRAM_MAX_ADR]
//--------------------------------------------------------------
void UB_SRAM_Write(uint32_t adr, uint16_t wert)
{
  // adresse muss um 1bit nach links verschoben werden
  adr=adr<<1;

  *(uint16_t *) (SRAM_ADR + adr) = wert;
}


//--------------------------------------------------------------
// liet einen 16bit Wert aus dem externe SRAM
// von einer Adresse
// ret_wert : [0 bis 65535]
// adr  : [0 bis SRAM_MAX_ADR]
//--------------------------------------------------------------
uint16_t UB_SRAM_Read(uint32_t adr)
{
  uint16_t ret_wert=0;

  // adresse muss um 1bit nach links verschoben werden
  adr=adr<<1;

  ret_wert=*(__IO uint16_t*) (SRAM_ADR + adr);

  return(ret_wert);
}



//--------------------------------------------------------------
// interne Funktion
// Init aller IO-Pins fuer das SRAM
//--------------------------------------------------------------
void P_SRAM_InitIO(void)
{

  GPIO_InitTypeDef GPIO_InitStructure;

  //-----------------------------------------
  // Clock Enable von Port-B, Port-D und Port-E
  //-----------------------------------------
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB,ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD,ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE,ENABLE);

  //-----------------------------------------
  // Alle Pins von Port-B initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource7,  GPIO_AF_FSMC); // PB7=FSMC_NL   -> NL

  // Structur fuer Port-B
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
  
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
  // Config von Port-B
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  //-----------------------------------------
  // Alle Pins von Port-D initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource0,  GPIO_AF_FSMC); // PD0=FSMC_D2   -> AD2
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource1,  GPIO_AF_FSMC); // PD1=FSMC_D3   -> AD3
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource4,  GPIO_AF_FSMC); // PD4=FSMC_NOE  -> OE
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource5,  GPIO_AF_FSMC); // PD5=FSMC_NWE  -> WE
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource7,  GPIO_AF_FSMC); // PD7=FSMC_NE1  -> CE
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource8,  GPIO_AF_FSMC); // PD8=FSMC_D13  -> AD13
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource9,  GPIO_AF_FSMC); // PD9=FSMC_D14  -> AD14
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource10, GPIO_AF_FSMC); // PD10=FSMC_D15 -> AD15
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource11, GPIO_AF_FSMC); // PD11=FSMC_A16 -> A16
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource12, GPIO_AF_FSMC); // PD12=FSMC_A17 -> A17
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource14, GPIO_AF_FSMC); // PD14=FSMC_D0  -> AD0
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource15, GPIO_AF_FSMC); // PD15=FSMC_D1  -> AD1

  // Structur fuer Port-D
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4 | GPIO_Pin_5 |
                                GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 |
                                GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_14 | GPIO_Pin_15;
  
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
  // Config von Port-D
  GPIO_Init(GPIOD, &GPIO_InitStructure); 

  //-----------------------------------------
  // Alle Pins von Port-E initialisieren
  //-----------------------------------------  
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource7,  GPIO_AF_FSMC); // PE7=FSMC_D4   -> AD4
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource8,  GPIO_AF_FSMC); // PE8=FSMC_D5   -> AD5  
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource9,  GPIO_AF_FSMC); // PE9=FSMC_D6   -> AD6
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource10, GPIO_AF_FSMC); // PE10=FSMC_D7  -> AD7
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource11, GPIO_AF_FSMC); // PE11=FSMC_D8  -> AD8
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource12, GPIO_AF_FSMC); // PE12=FSMC_D9  -> AD9
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource13, GPIO_AF_FSMC); // PE13=FSMC_D10 -> AD10
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource14, GPIO_AF_FSMC); // PE14=FSMC_D11 -> AD11
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource15, GPIO_AF_FSMC); // PE15=FSMC_D12 -> AD12
  
  // Structur fuer Port-E
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | 
                                GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;                                  
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
  // Config von Port-E
  GPIO_Init(GPIOE, &GPIO_InitStructure);

}


//--------------------------------------------------------------
// interne Funktion
// Init vom FSMC fuer das SRAM (im Mode-2)
// CS an PD7=FSMC_NE1 = Bank-1 / PSRAM-1
//--------------------------------------------------------------
void P_SRAM_InitFSMC(void)
{
  FSMC_NORSRAMInitTypeDef        FSMC_NORSRAMInitStructure;
  FSMC_NORSRAMTimingInitTypeDef  FSMC_NORSRAMTimingInitStructure;
  
  //-----------------------------------------
  // Clock Enable von FSMC
  //-----------------------------------------
  RCC_AHB3PeriphClockCmd(RCC_AHB3Periph_FSMC, ENABLE);  
  
  //-----------------------------------------
  // Structure fuer das Timing
  //-----------------------------------------  
  FSMC_NORSRAMTimingInitStructure.FSMC_AddressSetupTime = SRAM_FSMC_AST;
  FSMC_NORSRAMTimingInitStructure.FSMC_AddressHoldTime = SRAM_FSMC_AHT;
  FSMC_NORSRAMTimingInitStructure.FSMC_DataSetupTime = SRAM_FSMC_DST;
  FSMC_NORSRAMTimingInitStructure.FSMC_BusTurnAroundDuration = 0;
  FSMC_NORSRAMTimingInitStructure.FSMC_CLKDivision = 0;
  FSMC_NORSRAMTimingInitStructure.FSMC_DataLatency = 0;
  FSMC_NORSRAMTimingInitStructure.FSMC_AccessMode = FSMC_AccessMode_A;

  //-----------------------------------------
  // Structure fuer Bank-1 / PSRAM-1
  // (Mode-2, Multiplex enable)
  //-----------------------------------------  
  FSMC_NORSRAMInitStructure.FSMC_Bank = FSMC_Bank1_NORSRAM1;
  FSMC_NORSRAMInitStructure.FSMC_DataAddressMux = FSMC_DataAddressMux_Enable;
  FSMC_NORSRAMInitStructure.FSMC_MemoryType = FSMC_MemoryType_NOR;
  FSMC_NORSRAMInitStructure.FSMC_MemoryDataWidth = FSMC_MemoryDataWidth_16b;
  FSMC_NORSRAMInitStructure.FSMC_BurstAccessMode = FSMC_BurstAccessMode_Disable;
  FSMC_NORSRAMInitStructure.FSMC_AsynchronousWait = FSMC_AsynchronousWait_Disable;   
  FSMC_NORSRAMInitStructure.FSMC_WaitSignalPolarity = FSMC_WaitSignalPolarity_Low;  
  FSMC_NORSRAMInitStructure.FSMC_WrapMode = FSMC_WrapMode_Disable;  
  FSMC_NORSRAMInitStructure.FSMC_WaitSignalActive = FSMC_WaitSignalActive_BeforeWaitState;  
  FSMC_NORSRAMInitStructure.FSMC_WriteOperation = FSMC_WriteOperation_Enable;  
  FSMC_NORSRAMInitStructure.FSMC_WaitSignal = FSMC_WaitSignal_Disable;   
  FSMC_NORSRAMInitStructure.FSMC_ExtendedMode = FSMC_ExtendedMode_Disable;  
  FSMC_NORSRAMInitStructure.FSMC_WriteBurst = FSMC_WriteBurst_Disable;  
  FSMC_NORSRAMInitStructure.FSMC_ReadWriteTimingStruct = &FSMC_NORSRAMTimingInitStructure; 
  FSMC_NORSRAMInitStructure.FSMC_WriteTimingStruct = &FSMC_NORSRAMTimingInitStructure;
  
  // Config vom FSMC
  FSMC_NORSRAMInit(&FSMC_NORSRAMInitStructure); 
  
  // Enable Bank-1 / PSRAM-1
  FSMC_NORSRAMCmd(FSMC_Bank1_NORSRAM1, ENABLE);

}


